package net.jolsrv2;

import java.net.InetAddress;
/**
 * The class encapsulates an IP address and a prefix length, acting as a CIDR address.
 * @author Andreas Schjnhaug
 *
 */
public class CidrAddress{
	
	private InetAddress address;
	private int prefixLength;
	
	/**
	 * Constructor
	 * @param address The address
	 * @param prefixLength The prefix length
	 */
	public CidrAddress(InetAddress address, int prefixLength){
		this.address = address;
		this.prefixLength = prefixLength;
	}
	/**
	 * Alternative constructor, taking only the address. The prefix length will be the length of the address, in bits.
	 * @param address The address
	 */
	public CidrAddress(InetAddress address){
		this.address = address;
		this.prefixLength = address.getAddress().length * 8;
	}
	

	/**
	 * The method returns the address
	 * @return address The address
	 */
	public InetAddress getAddress(){
		return address;
		
	}
	/**
	 * The method returns the prefix length
	 * @return prefixLength The prefix length
	 */
	public int getPrefixLength(){
		return prefixLength;
	}
	/**
	 * The method sets the prefix length
	 * @param prefixLength The prefix length
	 */
	public void setPrefixLength(int prefixLength){
		this.prefixLength = prefixLength;
	}
	/**
	 * The method returns a string containing the address and prefix length split by a "/", for readability.
	 * Example: 192.168.10.100/32
	 * @return string
	 */
	public String toString(){
		return new String(address.getHostAddress() + "/" + prefixLength);
	}
	
	/**
	 * The method checks to see if two CidrAddresses are equals, that is, if both the address and prefix length have the same values.
	 */
	
	public boolean equals(Object cidrBlock){
		if(cidrBlock instanceof CidrAddress)
			return(((CidrAddress)cidrBlock).getAddress().equals(address) && ((CidrAddress)cidrBlock).getPrefixLength() == prefixLength);
		return false;
	}

}
