package net.jolsrv2;

import java.net.DatagramPacket;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.nio.ByteBuffer;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.logging.Logger;

import net.jolsrv2.constants.MessageTypes;
import net.jolsrv2.constants.TlvTypes;
import net.jolsrv2.data.AddressBlock;
import net.jolsrv2.data.Message;
import net.jolsrv2.data.ParsedPacket;
import net.jolsrv2.data.Tlv;
import net.jolsrv2.data.TlvBlock;
import net.jolsrv2.repository.processforward.ProcessedSet;
import net.jolsrv2.utils.Mantissa;
/**
 * The class parses incoming packets.
 * @author Andreas Schjnhaug
 *
 */
public final class PacketParser {
	
	private static Logger logger = Logger.getLogger(PacketParser.class.toString());
	
	private static PacketParser ref;
	
	private ByteBuffer buffer;
	
	private byte reserved = 0;
	private byte zero = 0;
	
	/**
	 * Constructor
	 *
	 */
	private PacketParser(){
	}
	
	/**
	 * Implementation of the singleton design pattern
	 * @return the packet parser
	 */
	public static synchronized PacketParser getPacketParser(){
		if(ref == null)
			ref = new PacketParser();
		return ref;
	}
	/**
	 * Implementation of the singleton design pattern. This method is used to
	 * ensure that the class can not be cloned.
	 */
	public Object clone() throws CloneNotSupportedException{
		throw new CloneNotSupportedException();
	}

	/**
	 * The method takes an incoming packet on a given interface and parses it.
	 * @param datagramPacket the incoming packet
	 * @param iface the interface that received the packet
	 */
	public ParsedPacket parsePacket (DatagramPacket datagramPacket, Interface iface) throws ParseException{
		
		
		
		
		ArrayList<Message> messages = new ArrayList<Message>();

		InetAddress sendingAddress = datagramPacket.getAddress();
		
		
		
		int ipLength = Node.getNode().getIpVersion();
		
		
		

		//byte[] bytes = ;
		buffer = ByteBuffer.wrap(datagramPacket.getData());
		//TODO: What if length is 0?		
		
		

		logger.fine("");
		logger.fine(",------------------");
		logger.fine("|  PARSING PACKET");
		logger.fine("|------------------");
		logger.fine("| * Receiving interface: " + iface.getLocalCAddress());

		

		if ((buffer.get() & 0xFF) == zero && (buffer.get() & 0xFF) == reserved){
			logger.fine("| - Found zero and reserved");
			int psn = buffer.getShort() & 0xFFFF;
			logger.fine("| * Packet seq number: " + psn);
		}
		
		else{
			throw new ParseException("No packet header found",buffer.position());
			
		}
		
		
		int endOfMessage = buffer.position();
		
		int temp = buffer.get() & 0xFF;
		//Looping over the messages in the packet
		while(temp != 0){
		
			logger.fine("|    ,-------------------");
			logger.fine("|    |  PARSING MESSAGE  ");
			logger.fine("|    |-------------------");
			
			Message message = new Message();
			
			MessageTypes type = null;
			
			
			for(MessageTypes mt : MessageTypes.values()){
				if(mt.byteValue() == temp){
					type = mt;
					break;
				}
				
			}
			
			
			message.setType(type);
			logger.fine("|    | * Message type:       " + type);
			int msgSemantics = buffer.get() & 0xFF;
			message.setSemantics(msgSemantics);
			logger.fine("|    | * Message semantics : " + msgSemantics);
			int msgSize = buffer.getShort() & 0xFF;
			logger.fine("|    | * Message size:       " + msgSize);
			
			
			
			//msgSemantics == 0 means that the message includes msg-header-info
			if(msgSize < 1)
				throw new ParseException("negative message size", buffer.position());
			if(msgSemantics == 0){
				
				endOfMessage += msgSize;
			
				//TODO: Check if mesasge contains fragments
				
				//Reading msg-header-info
				InetAddress originatorAddress = null;
				byte[] oa = new byte[ipLength];
				buffer.get(oa);
				try{
					originatorAddress = InetAddress.getByAddress(oa);
					logger.fine("|    | * Originator address: " + originatorAddress);
				}
				catch(UnknownHostException e){logger.warning("|    | - Malformed originator address, skipping message");}
				
				
				if(originatorAddress == null)
					buffer.position(endOfMessage);
				
				else if(originatorAddress.equals(iface.getLocalCAddress().getAddress())){
					//Skip the message
					logger.warning("|    | - Received message originated from this node, skipping messge.");
					buffer.position(endOfMessage); 
				}
				else{

					message.setOriginatorAddress(originatorAddress);
					
					int hopLimit = buffer.get() & 0xFF;
					if(hopLimit <= 0){
						//Skip the message
						logger.warning("Found hop limit <= 0, skipping.");
						buffer.position(endOfMessage); 
					}
					else{
						message.setHopCount(hopLimit);
					
						logger.fine("|    | * Hop limit:          " + hopLimit);
						int hopCount = buffer.get() & 0xFF;
						message.setHopCount(hopCount);
						logger.fine("|    | * Hop count:          " + hopCount);

						int msgSeqNumber = buffer.getShort() & 0xFFFF;
						message.setSequenceNumber(msgSeqNumber);
						logger.fine("|    | * Message seq number: " + msgSeqNumber);
	
	
						
						//Adds the message TLVs to the Message
						TlvBlock messageTlvBlock = parseTlvBlock();
						
						//Checks to see if we can find Validity time which MUST be present
						long validityTime = -1;
						
						for(Tlv tlv : messageTlvBlock.getTlvs()){
							if(tlv.getType().equals(TlvTypes.VALIDITY_TIME) && tlv.getValue().length == 1){
								validityTime = Mantissa.decode(tlv.getValue()[0]);
							}
							
							
							
						}
						if(validityTime == -1){
							logger.warning("|       | - Validity time not found, skipping message");
							buffer.position(endOfMessage);
							
						}
						else{
							
							
							
							message.setMessageTlvBlock(messageTlvBlock);
							
							//Finished reading header fields
							
						
							
							//--------------
							//  PROCESSING
							//--------------
							if(type.equals(MessageTypes.HELLOV2) || type.equals(MessageTypes.TCV2)){
								//Message is considered for processing
								
	
								//Checking whether this message has already been processed
								if(type.equals(MessageTypes.TCV2) && ProcessedSet.getProcessedSet().contains(type, originatorAddress, msgSeqNumber)){
								
									//Skipping the message
									logger.warning("|    | - Message already processed, skipping..");
									buffer.position(buffer.position() + msgSize - ipLength - 8);
								}
								else{
									
									if(type.equals(MessageTypes.TCV2))
										ProcessedSet.getProcessedSet().addTuple(type, originatorAddress, msgSeqNumber, System.nanoTime() + 10*(long)1e9); //TODO: What is P_HOLD_TIME??
								
									
									//Check to see if there will be address blocks/tlv blocks							
									
									while(buffer.position() < endOfMessage){
									
									
										buffer.mark();
										if ((buffer.getShort()& 0xFFFF) != 0){
											buffer.reset();
											
											
											//TODO: If this is a HELLO message, then it must be checked that LINK STATUS tlvs exists, see chapter 8 i the OLSRv2 specification
											try{

												message.addAddressBlock(parseAddressBlock());
												message.addTlvBlock(parseTlvBlock());
											}
											catch(ParseException e){throw e;}	
										}	
									}
									//Finished parsing message
									messages.add(message);
									
									
									
									//populateRepositories(message, iface.getLocalCB());
									
		
								}					
							}					
						}
					}
				}
			}
			
			logger.fine("|    `------------------");
			
			
			//Reading the next byte to see if it's not zero
			temp = buffer.get() & 0xFF;
		}
		
		logger.fine("`-----------------");
		
		return new ParsedPacket(messages, sendingAddress);
	}
	/**
	 * The method parses an address block
	 * @return a parsed address block
	 */
	public AddressBlock parseAddressBlock() throws ParseException{
		logger.fine("|    | - Parsing address block");
		AddressBlock addressBlock = new AddressBlock();
		int ipVersion = Node.getNode().getIpVersion();
		
			  
		int numAddresses =  buffer.get() & 0xFF;
		if(numAddresses < 1)
			  throw new ParseException("Non-postive or zero addresses found in address block", buffer.position());
		
		int headOctet = buffer.get() & 0xFF;
		if(headOctet < 0)
			throw new ParseException("Negative head octet found in address block", buffer.position());

		int headLength = headOctet >> 1;
		byte[] head = null;
	    if(headLength != 0){
	    	head = new byte[headLength];
	    	buffer.get(head);
	    }
	      
	    int tailOctet = 0;
	    int tailLength = 0;
	    byte[] tail = null;
	      
	      
	    if((headOctet & 0x1) == 0){
	    	  
	    	tailOctet = buffer.get() & 0xFF;
	    	tailLength = tailOctet >> 1;
		    if(tailLength > 0){
		    	tail = new byte[tailLength];
		    	buffer.get(tail);
		    	  
		    }
		    else if(tailLength < 0)
		    	throw new ParseException("negative tail length",buffer.position());
	    }
	      
	    int midLength = ipVersion - headLength - tailLength;
	    byte[] mid = null;
	    if(midLength > 0){
	    	mid = new byte[midLength * numAddresses];
	    	buffer.get(mid);
	    	  
	    }
	    else if(midLength < 0)
	    	throw new ParseException("negative mid length",buffer.position());
	
		for(int i = 0; i < numAddresses ; i++){
			  
			byte[] bAddress = new byte[ipVersion];
			int pointer = 0;
			//header
			if(head != null)
	    		for(int j = 0; j < head.length; j++)
	    			bAddress[pointer++] = head[j];
			  
			//mid
			if(mid != null)
	    		for(int k = i*midLength ; k < (i*midLength) + midLength; k++){
	    			bAddress[pointer++] = mid[k];
	    		}
			//tail
			if((tailOctet & 0x1) == 1){
				for(; pointer < ipVersion ;)
					bAddress[pointer++] = 0;
				  
			}
			else if(tail != null)
				for(int j = 0; j < tail.length; j++)
					bAddress[pointer++] = tail[j];
			try{
				InetAddress address = InetAddress.getByAddress(bAddress);
				logger.finer("|    |     - Found: " + address);
				if(addressBlock.getAddresses().contains(address))
					throw new ParseException("Duplicate address found",buffer.position());
				else
					addressBlock.add(address);
			}
			catch(UnknownHostException e){throw new ParseException("Malformed address", buffer.position());}
			
			  
		}
		
		return (addressBlock);
	}
	
	/**
	 * Method for parsing TLV blocks
	 * @return the parsed TLV block
	 */
	private TlvBlock parseTlvBlock(){
		
		logger.fine("|    | - Parsing TLV block");
		//Rebuilding the TLVs contained in a TlvBlock
		ArrayList<Tlv> tlvs = new ArrayList<Tlv>();
		
		int tlvLength = buffer.getShort() & 0xFFFF;
	
		
		int readUntil = buffer.position() + tlvLength;
	
		while(buffer.position() < readUntil){
		
			
			int b = buffer.get() & 0xFF;
			
			TlvTypes tlvType = null;
			
			for(TlvTypes t : TlvTypes.values()){
				if(t.byteValue() == b)
					tlvType = t;
				
			}
			
			
			if(tlvType == null)
				logger.warning("TODO: Tlv type not found");
			
			int tlvSemantics = buffer.get() & 0xFF;
			
			
			Tlv tlv = new Tlv(tlvType);
			tlv.setSemantics(tlvSemantics);
			
			int length = 0;
			if((tlvSemantics & 0x1) == 0){
				
				if ((tlvSemantics >> 1 & 0x1) == 0)
					 length = buffer.get() & 0xFF;
				else if ((tlvSemantics >> 1 & 0x1) == 1)
					 length = buffer.getShort() & 0xFFFF;
				
				
			}
			
			if((tlvSemantics >> 2 & 0x1) == 0){
				tlv.setIndexStart(buffer.get() & 0xFF);
				tlv.setIndexStop(buffer.get() & 0xFF);
				
			}
			
			
			
			if((tlvSemantics & 0x1) == 0){
				byte[] value = new byte[length];
				buffer.get(value);
				tlv.setValue(value);				
			}
			
			tlvs.add(tlv);
		
		}
		
		return new TlvBlock(tlvs);
		
	}
}
