package net.jolsrv2.comm;

import java.net.*;
import java.text.ParseException;
import java.util.logging.Logger;
import java.io.*;


import net.jolsrv2.*;
import net.jolsrv2.data.ParsedPacket;
/**
 * The class listens on an Interface's DatagramSocket for incoming packets.
 * The packets are received, parsed and put in the queue for parsed packets
 * on the Interface.
 * @author Andreas Schjnhaug
 *
 */
public class Receiver implements Runnable{
	
	private static Logger logger = Logger.getLogger(Receiver.class.toString());
	
	
	private DatagramSocket socket;
	private boolean running = true;

	private Interface iface;
	
	/*
	 * The following variables are used to create different topologies when
	 * several computers are within transmission range of each other, and their
	 * connections are not desired.
	 * 
	
	private InetAddress laptop;
	private InetAddress pc1;
	private InetAddress pc2;
	private InetAddress pc3;
	private InetAddress pc4;*/
	
	
	
	/**
	 * Constructor
	 * @param iface the belonging Interface
	 */
	public Receiver(Interface iface){
		this.iface = iface;
		socket = iface.getSocket();
		
		
		
		
		/*
		 * The following variables are used to create different topologies when
		 * several computers are within transmission range of each other, and their
		 * connections are not desired.
		 * 
		 * The nodes' IP addresses must all be known to perform filtering.
		 * 
		try{
			laptop   = InetAddress.getByName("192.168.10.100");
			pc1      = InetAddress.getByName("192.168.10.21");
			pc2 	 = InetAddress.getByName("192.168.10.22");
			pc3      = InetAddress.getByName("192.168.10.23");
			pc4      = InetAddress.getByName("192.168.10.24");
			
		}
		catch(Exception e){Sout.err(e);}*/
	
		
	}
	
	public void halt(){
		running = false;
	}
	/**
	 * Definition of the run method as required by the Runnable interface
	 * The method creates a loop listening for packet, trying to parse them
	 * as they arrive. The successfully parsed packets are put in the
	 * respective Interface's parsed packets queue.
	 * @see PacketParser
	 * @see Interface
	 */
	public void run(){
		
		
		while(running){
			
			byte[] buffer = new byte[65507];
			
			DatagramPacket dp = new DatagramPacket(buffer, buffer.length);
			
			try{
				socket.receive(dp);
				
				

				/**
				 * The following variables are used to create different topologies when
				 * several computers are within transmission range of each other, and their
				 * connections are not desired.
				
				
				
				//Example topology
				
				
				/*
				 *  PC1
				 *     \
				 *      laptop
				 *     /      \
				 *  PC3        PC4
				 * 
				 */
				/*
				boolean ok = false;
				// LAPTOP <-> PC1
				if(iface.getAddress().equals(laptop) && dp.getAddress().equals(pc1))
					ok = true;
				else if(iface.getAddress().equals(pc1) && dp.getAddress().equals(laptop))
					ok = true;
				
				
				//LAPTOP <-> PC3
				if(iface.getAddress().equals(laptop) && dp.getAddress().equals(pc3))
					ok = true;
				else if(iface.getAddress().equals(pc3) && dp.getAddress().equals(laptop))
					ok = true;
				
				//LAPTOP <-> PC4
				if(iface.getAddress().equals(laptop) && dp.getAddress().equals(pc4))
					ok = true;
				else if(iface.getAddress().equals(pc4) && dp.getAddress().equals(laptop))
					ok = true;
				
				
				*/
				
				
				
				//Example topology
				
				
				/*
				 *  PC1
				 *     \
				 *      laptop
				 *     /      
				 *  PC3 --------PC4
				 * 
				 */
				
				/*boolean ok = false;
				// LAPTOP <-> PC1
				if(iface.getAddress().equals(laptop) && dp.getAddress().equals(pc1))
					ok = true;
				else if(iface.getAddress().equals(pc1) && dp.getAddress().equals(laptop))
					ok = true;
				
				
				//LAPTOP <-> PC3
				if(iface.getAddress().equals(laptop) && dp.getAddress().equals(pc3))
					ok = true;
				else if(iface.getAddress().equals(pc3) && dp.getAddress().equals(laptop))
					ok = true;
				
				//PC3 <-> PC4
				if(iface.getAddress().equals(pc3) && dp.getAddress().equals(pc4))
					ok = true;
				else if(iface.getAddress().equals(pc4) && dp.getAddress().equals(pc3))
					ok = true;
				
				
				
				*/
				
				boolean ok = true;
				
				
				
				//Filtering of packets:
				
				if (ok && !dp.getAddress().equals(iface.getLocalCAddress().getAddress())){
					
					logger.info("Packet received on interface " + iface.getLocalCAddress() + " from " + dp.getAddress());
					try{
						if(dp.getAddress().getAddress().length != Node.getNode().getIpVersion())
							throw new ParseException("Incoming packet uses different IP version.",0);
						
						ParsedPacket parsedPacket = PacketParser.getPacketParser().parsePacket(dp,iface);
					
						iface.offerToParsedPackets(parsedPacket);
					}
					catch(ParseException e){logger.warning("Rest of the packet dumped due to " + e);}
				}
			}
			catch(IOException e){logger.warning(e.toString());}
			
		}
		
	}

}
