package net.jolsrv2.comm;

import java.net.*;
import java.util.*;
import java.util.logging.Logger;

import net.jolsrv2.*;
import net.jolsrv2.data.*;
/**
 * The class looks for outgoing messages on an Interface,
 * packs as many as possible into a Packet and sends it
 * over a DatagramSocket.
 * @author Andreas Schjnhaug 
 *
 */
public class Sender implements Runnable{
	
	private static Logger logger = Logger.getLogger(Sender.class.toString());
	
	private DatagramSocket socket;
	private boolean running = true;
	private Interface iface;
	InetSocketAddress broadcast;
	/**
	 * Constructor 
	 * @param iface the belonging Interface
	 * @param broadcast the broadcast SocketAddress
	 */
	public Sender(Interface iface, InetSocketAddress broadcast){
		
		this.iface = iface;
		this.broadcast = broadcast;
		
		socket = iface.getSocket();
	}
	
	/**
	 * Definition of the run method as required by the Runnable interface
	 * The method loops, listening for outgoing messages in the belonging
	 * Interface. When there one or more messags in the outgoing messages 
	 * queue, the method packets as many of them as possible into a Packet
	 * and sends it over the Interface's DatagramSocket
	 * @see Interface
	 */	
	public void run(){
		byte[] buffer = null;
		
		while(running){	
			ArrayList<Message> messages = iface.takeFromOutgoingMessages();
			
			Packet packet = new Packet(messages);
			packet.setSequenceNumber(iface.getAPacketSequenceNumber());
			
			try{
				buffer = packet.getBytes();
			}
			catch(Exception e){logger.warning("ERROR in Sender (1): " + e);}
			
			
			try{
				DatagramPacket dp = new DatagramPacket(buffer, buffer.length, broadcast);
				socket.send(dp);
				
				logger.info("Packet " + packet.getSequenceNumber() + " sent with " + buffer.length + " bytes containing " + messages.size() + " message(s) over interface " + iface.getLocalCAddress());
			}
			catch(Exception e){logger.warning("ERROR in Sender (2): " + e);}
			

		}
	}
}
