package net.jolsrv2.constants;
/**
 * The class is defining the TLV values used in OLSRv2, and their numeric values.
 * The values of these constants are at the time of writing listed as TBD 
 * in the Internet-Draft. 
 * @author Andreas Schjnhaug 
 *
 */
public enum Values {
	

	
	//Willingness
	WILL_NEVER(0),
	WILL_LOW(1),
	WILL_DEFAULT(3),
	WILL_HIGH(6),
	WILL_ALWAYS(7),
	
	//Link status
	HEARD(30),
	SYMMETRIC(20),
	LOST(10),
	
	
	//Holding times
	L_HOLD_TIME(3 * 2),
	
	//Intervals
	HELLO_INTERVAL(2),
	REFRESH_INTERVAL(2),
	TC_INTERVAL(5),
	
	//Values for MPR
	TRUE(1);
	

	  
	private final byte value;
	
	/**
	 * Constructor
	 * @param value of the TLV value
	 */
	Values(int value){
		this.value = (byte)value;
	}
	
	/**
	 * Returns the byte value of a TLV value
	 * @return the byte value
	 */
	public byte byteValue(){
		return value;
	}
	
	
}
