package net.jolsrv2.data;

import java.nio.ByteBuffer;
import java.util.*;
/**
 * The class implements the packet, as specified in OLSRv2.
 * @author Andreas Schjnhaug
 *
 */
public class Packet {
	
	private static final int zero = 0;
	private static final int reserved = 0;
	private int sequenceNumber;
	private ArrayList<Message> messages;

	/**
	 * Constructor
	 * @param messages the messages to be added to the packet
	 */
	public Packet(ArrayList<Message> messages){
		this.messages = messages;
	}
	
	/**
	 * Returns the packet sequence number
	 * @return the packet sequence number
	 */
	public int getSequenceNumber(){
		return sequenceNumber;
	}
	
	/**
	 * Sets the packet sequence number
	 * @param sequenceNumber the packet sequence number
	 */
	public void setSequenceNumber(int sequenceNumber){
		this.sequenceNumber = sequenceNumber;
	}
	
	 /**
	  * Returns the bytes of the packet
	  * @return the bytes of the packet
	  */
	public byte[] getBytes(){
		int size = 4;
		
		ArrayList<byte[]> messageBytes = new ArrayList<byte[]>();
		for(Message message : messages){
			byte[] bytes = message.getBytes();
			messageBytes.add(bytes);
			size += bytes.length;
		}
		
		byte[] bytes = new byte[size];
		ByteBuffer byteBuffer = ByteBuffer.wrap(bytes);
		
		
		
		byteBuffer.put((byte)zero);
		byteBuffer.put((byte)reserved);
		byteBuffer.putShort((short)sequenceNumber);
		for(byte[] bytess : messageBytes){
			byteBuffer.put(bytess);
		}
		
		return bytes;
	}
	
	
	
}

