package net.jolsrv2.data;

import java.nio.ByteBuffer;
import java.util.*;
/**
 * The class implements the TLV block, as specified in OLSRv2.
 * @author Andreas Schjnhaug
 *
 */
public class TlvBlock{
	
	
	private ArrayList<Tlv> tlvs;

	/**
	 * Empty constructor
	 *
	 */
	public TlvBlock(){
		tlvs = new ArrayList<Tlv>();
	}
	
	/**
	 * Constructor
	 * @param tlvs the TLVs to be put in the TLV block
	 */
	public TlvBlock(ArrayList<Tlv> tlvs){
		this.tlvs = tlvs;
	}
		
	/**
	 * Adds a TLV to the TLV block
	 * @param tlv the TLV
	 */
	public void add(Tlv tlv){
		tlvs.add(tlv);
		
	}
	
	/**
	 * Returns the TLVs in the TLV block
	 * @return the TLVs
	 */
	public ArrayList<Tlv> getTlvs(){
		return tlvs;
	}
	
	
	 /**
	  * Returns the bytes of the TLV block
	  * @return the bytes of the TLV block
	  */
	public byte[] getBytes(){
		
		ArrayList<byte[]> tlvBytes = new ArrayList<byte[]>();
		
		int size = 2; //tlvLength
		for(Tlv tlv : tlvs){
			byte[] tlvB = tlv.getBytes();
			tlvBytes.add(tlvB);
			size += tlvB.length;
		}
		byte[] bytes = new byte[size];
		ByteBuffer byteBuffer = ByteBuffer.wrap(bytes);
		
			
		byteBuffer.putShort((short)(size - 2));
		for(byte[] tlvB : tlvBytes){
			byteBuffer.put(tlvB);
		}
	
		return bytes;
		
	}
	
	
}
