package net.jolsrv2.repository.neighbourhood;

import net.jolsrv2.CidrAddress;
import net.jolsrv2.constants.Values;


/**
 * The class implements the symmetric neighbour tuple, as specified in OLSRv2.
 * @author Andreas Schjnhaug
 *
 */
public class SymmetricNeighbourTuple {
	
	
	private CidrAddress localCAddress;
	private CidrAddress neighbourCAddress;
	private long symTime;
	private long time;
	
	/**
	 * Constructor
	 * @param localCAddress the local CIDR address
	 * @param neighbourCAddress the neighbour CIDR address
	 * @param symTime the symmetric time
	 * @param time the time
	 */
	public SymmetricNeighbourTuple(CidrAddress localCAddress, CidrAddress neighbourCAddress, long symTime, long time){
		this.localCAddress = localCAddress;
		this.neighbourCAddress = neighbourCAddress;
		this.symTime = symTime;
		this.time = time;
	}
	
	/**
	 * Returns the local CIDR address
	 * @return the local CIDR address
	 */
	public CidrAddress getLocalCAddress(){
		return localCAddress;
	}
	
	/**
	 * Returns the neighbour CIDR address
	 * @return the neighbour CIDR address
	 */
	public CidrAddress getNeighbourCAddress(){
		return neighbourCAddress;
	}
	
	/**
	 * Returns the symmetric time
	 * @return the symmetric time
	 */
	public long getSymTime(){
		return symTime;
	}
	
	/**
	 * Sets  the symmetric time
	 * @param symTime the symmetric time
	 */
	public void setSymTime(long symTime){
		this.symTime = symTime;
	}
	
	/**
	 * Returns the time
	 * @return the time
	 */
	public long getTime(){
		return time;
	}
	
	
	/**
	 * Sets the time
	 * @param time the time
	 */
	public void setTime(long time){
		this.time = time;
	}
	
	/**
	 * Returns the status
	 * @return the status, in form of a Values object either being SYMMETRIC or LOST
	 */
	public Values getStatus(){
		
		if(symTime > System.nanoTime())
			return Values.SYMMETRIC;
		else
			return Values.LOST;
		
	}
	
	

}
