package net.jolsrv2.repository.neighbourhood;

import net.jolsrv2.CidrAddress;
/**
 * The class implements the two hop tuple, as specified in OLSRv2.
 * @author Andreas Schjnhaug
 *
 */
public class TwoHopTuple {
	
	private CidrAddress localCAddress;
	private CidrAddress neighbourCAddress;
	private CidrAddress twoHopCAddress;
	private long time;
	
	/**
	 * Constructor
	 * @param localCAddress the local CIDR address
	 * @param neighbourCAddress the neighbour CIDR address
	 * @param twoHopCAddress the two hop CIDR address
	 * @param time the time
	 */
	public TwoHopTuple(CidrAddress localCAddress, CidrAddress neighbourCAddress, CidrAddress twoHopCAddress,long time){
		this.localCAddress = localCAddress;
		this.neighbourCAddress = neighbourCAddress;
		this.twoHopCAddress = twoHopCAddress;
		this.time = time;
	}
	
	/**
	 * Returns the local CIDR address
	 * @return the local CIDR address
	 */
	public CidrAddress getLocalCAddress(){
		return localCAddress;
	}
	
	/**
	 * Returns the neighbour CIDR address
	 * @return the neighbour CIDR address
	 */
	public CidrAddress getNeighbourCAddress(){
		return neighbourCAddress;
	}
	
	/**
	 * Return the two hop CIDR address
	 * @return the two hop CIDR address
	 */
	public CidrAddress getTwoHopCAddress(){
		return twoHopCAddress;
	}
	
	/**
	 * Returns the time
	 * @return the time
	 */
	public long getTime(){
		return time;
	}
	
	/**
	 * Sets the time
	 * @param time the time
	 */
	public void setTime(long time){
		this.time = time;
	}

}
