package net.jolsrv2.repository.processforward;

import java.net.InetAddress;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.logging.Logger;

import net.jolsrv2.constants.MessageTypes;

/**
 * The class implements the processed set, as specified in OLSRv2.
 * @author Andreas Schjnhaug
 *
 */
public final class ProcessedSet {
	
	private static Logger logger = Logger.getLogger(ProcessedSet.class.toString());
	
	private static ProcessedSet ref;
	
	private ArrayList<ProcessedTuple> tuples;
	
	/**
	 * Constructor
	 *
	 */
	public ProcessedSet(){
		tuples = new ArrayList<ProcessedTuple>();
	}
	/**
	 * Implementation of the singleton design pattern
	 * @return the processed set
	 */
	public static synchronized ProcessedSet getProcessedSet(){
		if(ref == null)
			ref = new ProcessedSet();
		return ref;
	}
	
	/**
	 * Implementation of the singleton design pattern. This method is used to
	 * ensure that the class can not be cloned.
	 */
	public Object clone() throws CloneNotSupportedException{
		throw new CloneNotSupportedException();
	}
	
	/**
	 * Checks if the processed set contains a processed tuple with a certain message type, IP address and sequence number
	 * @param type the message type
	 * @param address the IP address
	 * @param sequenceNumber the sequence number
	 * @return true if the processed tuple exists
	 */
	public boolean contains(MessageTypes type, InetAddress address, int sequenceNumber){
		
		for(ProcessedTuple tuple : tuples){
			if(tuple.getType().equals(type) && tuple.getAddress().equals(address) && tuple.getSequenceNumber() == sequenceNumber)
				return true;
		}
		return false;
		
	}
	
	/**
	 * Adds a tuple to the processed set
	 * @param type the message type
	 * @param address the IP address
	 * @param sequenceNumber the sequence number
	 * @param time the time
	 */
	public void addTuple(MessageTypes type, InetAddress address, int sequenceNumber, long time){
		tuples.add(new ProcessedTuple(type, address, sequenceNumber, time));
	}
	/**
	 * The method purges the processed set, deleting expired entries
	 * 
	 */
	public void purge(){
		try{
			for(Iterator<ProcessedTuple> i = tuples.iterator(); i.hasNext(); ){
				ProcessedTuple tuple = i.next();
				if (tuple.getTime() < System.nanoTime()){
					i.remove();
					
				}
				
			}
		}
		catch(Exception e){logger.warning("ERROR in GeneralSet: " + e);}
	}

}
