package net.jolsrv2.repository.processforward;

import java.net.InetAddress;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.logging.Logger;

import net.jolsrv2.constants.MessageTypes;

/**
 * The class implements the received tuple, as specified in OLSRv2.
 * @author Andreas Schjnhaug
 *
 */
public class ReceivedSet {
	
	private static Logger logger = Logger.getLogger(ReceivedSet.class.toString());
	
	private ArrayList<ProcessedTuple> tuples;
	/**
	 * Constructor
	 *
	 */
	public ReceivedSet(){
		tuples = new ArrayList<ProcessedTuple>();
	}
	
	
	/**
	 * Checks if the received set contains a received tuple with having a certain message type, IP address and sequence number
	 * @param type the message type
	 * @param address the IP address
	 * @param sequenceNumber the sequence number
	 * @return true if a change has been detected
	 */
	public boolean contains(MessageTypes type, InetAddress address, int sequenceNumber){
		
		for(ProcessedTuple tuple : tuples){
			if(tuple.getType().equals(type) && tuple.getAddress().equals(address) && tuple.getSequenceNumber() == sequenceNumber)
				return true;
		}
		return false;
		
	}
	/**
	 * Adds a received tuple to the received tuple set
	 * @param type the message type
	 * @param address the IP address
	 * @param sequenceNumber the sequence number
	 * @param time the time
	 */
	public void addTuple(MessageTypes type, InetAddress address, int sequenceNumber, long time){
		tuples.add(new ProcessedTuple(type, address, sequenceNumber, time));
	}
	/**
	 * The method purges the received set, deleting expired entries
	 * 
	 */
	public void purge(){
		try{
			for(Iterator<ProcessedTuple> i = tuples.iterator(); i.hasNext(); ){
				ProcessedTuple tuple = i.next();
				if (tuple.getTime() < System.nanoTime()){
					i.remove();
					
				}
				
			}
		}
		catch(Exception e){logger.warning("ERROR in GeneralSet: " + e);}
	}

}
