package net.jolsrv2.repository.processforward;

import java.net.InetAddress;
import java.util.ArrayList;
/**
 * The class implements the relay set, as specified in OLSRv2.
 * @author Andreas Schjnhaug
 *
 */
public class RelaySet {
	
	private static RelaySet ref;
	
	private ArrayList<InetAddress> tuples;
	/**
	 * Constructor
	 *
	 */
	private RelaySet(){
		tuples = new ArrayList<InetAddress>();
		
	}
	/**
	 * Implementation of the singleton design pattern
	 * @return the relay set
	 */
	public static synchronized RelaySet getRelaySet(){
		if(ref == null)
			ref = new RelaySet();
		return ref;
	}
	
	/**
	 * Implementation of the singleton design pattern. This method is used to
	 * ensure that the class can not be cloned.
	 */
	public Object clone() throws CloneNotSupportedException{
		throw new CloneNotSupportedException();
	}
	/**
	 * Checks if the realy set contains a received tuple with having a certain IP address
	 * @param address
	 * @return true if the received tuple exists
	 */
	public boolean contains(InetAddress address){
		return tuples.contains(address);
	}
	
	/**
	 * The method deletes all relay tuples
	 *
	 */
	public void clear(){
		tuples.clear();
	}

	/**
	 * Adds an IP address to the relay set
	 * @param address IP address 
	 */
	public void add(InetAddress address){
		tuples.add(address);
	}

}
