package net.jolsrv2.repository.topology;

import java.net.InetAddress;
import java.util.ArrayList;

import net.jolsrv2.CidrAddress;
import net.jolsrv2.repository.neighbourhood.MprSelectorSet;
/**
 * The class implements the advertised neighbour set, as specified in OLSRv2.
 * @author Andreas Schjnhaug
 *
 */
public final class AdvertisedNeighbourSet {
	
	private static AdvertisedNeighbourSet ref;

	private ArrayList<InetAddress> addresses;
	private int ansn;
	
	/**
	 * Constructor
	 *
	 */
	private AdvertisedNeighbourSet(){
		addresses = new ArrayList<InetAddress>();
		ansn = 0;
	}
	
	/**
	 * Implementation of the singleton design pattern
	 * @return the advertised neighbour set
	 */
	public static synchronized AdvertisedNeighbourSet getAdvertisedNeighbourSet(){
		if(ref == null)
			ref = new AdvertisedNeighbourSet();
		return ref;
	}
	
	/**
	 * Implementation of the singleton design pattern. This method is used to
	 * ensure that the class can not be cloned.
	 */
	public Object clone() throws CloneNotSupportedException{
		throw new CloneNotSupportedException();
	}
	 
	/**
	 * Returns the advertised neighbour set sequence number
	 * @return the advertised neighbour set sequence number
	 */
	public int getAnsn(){
		return ansn;
	}
	
	/**
	 * Returns the IP addresses
	 * @return the IP addresses
	 */
	public ArrayList<InetAddress> getAddresses(){
		return addresses;
	}
	
	/**
	 * Method populating the advertsied neighbour set, as proposed in the specification
	 */
	public void populate(){
		ArrayList<CidrAddress> mprSelectors = MprSelectorSet.getMprSelectorSet().getCAddresses();
		addresses.clear();
		for(CidrAddress mprSelector : mprSelectors)
			addresses.add(mprSelector.getAddress());
		ansn++;
			
		
		
	}
}

