package net.jolsrv2.repository.topology;

import net.jolsrv2.CidrAddress;
/**
 * The class implements the attached network tuple, as specified in OLSRv2.
 * @author Andreas Schjnhaug
 *
 */
public class AttachedNetworkTuple {
	
	private CidrAddress networkCAddress;
	private CidrAddress gatewayCAddress;
	private int ansn;
	private long time;
	
	/**
	 * Constructor
	 * @param networkCAddress the network CIDR address
	 * @param gatewayCAddress the gateway CIDR address
	 * @param ansn the ANSN
	 * @param time the time
	 */
	public AttachedNetworkTuple(CidrAddress networkCAddress, CidrAddress gatewayCAddress,int ansn, long time){
		this.networkCAddress = networkCAddress;
		this.gatewayCAddress = gatewayCAddress;
		this.ansn = ansn;
		this.time = time;
	}
	
	/**
	 * Returns the network CIDR address
	 * @return the network CIDR address
	 */
	public CidrAddress getNetworkCAddress(){
		return networkCAddress;
	}
	
	/**
	 * Returns the gateway CIDR address
	 * @return the gateway CIDR address
	 */
	public CidrAddress getGatewayCAddress(){
		return gatewayCAddress;
	}
	
	/**
	 * Returns the ANSN
	 * @return the ANSN
	 */
	public int getAnsn(){
		return ansn;
	}
	
	/**
	 * Sets the ANSN
	 * @param ansn the ANSN
	 */
	public void setAnsn(int ansn){
		this.ansn = ansn;
	}
	

	
	/**
	 * Returns the time
	 * @return the time
	 */
	public long getTime(){
		return time;
	}	
	
	/**
	 * Sets the time
	 * @param time the time
	 */
	public void setTime(long time){
		this.time = time;
	}
	

	
}
