package net.jolsrv2.repository.topology;

import java.net.InetAddress;
/**
 * The class implements the routing tuple, as specified in OLSRv2.
 * @author Andreas Schjnhaug
 *
 */
public class RoutingTuple {
	
	private InetAddress destination;
	private InetAddress next;
	private int distance;
	private InetAddress localIfaceAddr;
	
	
	/**
	 * Constructor
	 * @param destination the destination IP address
	 * @param next the next IP address
	 * @param distance the ditance
	 * @param localIfaceAddr the local interface IP address
	 */
	public RoutingTuple(InetAddress destination, InetAddress next, int distance, InetAddress localIfaceAddr){
		this.destination = destination;
		this.next = next;
		this.distance = distance;
		this.localIfaceAddr = localIfaceAddr;
		
	}
	
	/**
	 * Returns the destination IP address
	 * @return the destination IP address
	 */
	public InetAddress getDestination(){
		return destination;
	}
	
	/**
	 * Return the next IP address
	 * @return the next IP address
	 */
	public InetAddress getNext(){
		return next;
	}
	
	
	/**
	 * Returns the distance
	 * @return the distance
	 */
	public int getDistance(){
		return distance;
	}
	
	/**
	 * Return the local interface IP address
	 * @return the local interface IP address
	 */
	public InetAddress getLocalIfaceAddr(){
		return localIfaceAddr;
	}
	
	/**
	 * Return the entry nicely formatted
	 * @return string the nicely formatted entry
	 */
	public String toString(){

		return new String("ADD " + destination.getHostAddress() + " " + next.getHostAddress() + " METRIC " + distance + " IF 0x33");
	}
	
	

}
