package net.jolsrv2.repository.topology;

import java.net.InetAddress;

/**
 * The class implements the topology set, as specified in OLSRv2.
 * @author Andreas Schjnhaug
 *
 */

public class TopologyTuple {

  private InetAddress destination;
  private InetAddress lastHop;
  private int ansn;
  private long time;
  
  /**
   * Constructor
   * @param destination the destination IP address
   * @param lastHop the IP address of the last hop
   * @param ansn the ANSN
   */
  public TopologyTuple(InetAddress destination, InetAddress lastHop, int ansn){
	  this.destination = destination;
	  this.lastHop = lastHop;
	  this.ansn = ansn;
  }
  
  /**
   * Returns the destination IP address
   * @return the destination IP address
   */
  public InetAddress getDestination(){
	  return destination;
  }
  
  /**
   * Returns the last hop
   * @return the last hop
   */
  public InetAddress getLastHop(){
	  return lastHop;
  }
  
  /**
   * Returns the ANSN
   * @return the ANSN
   */
  public int getAnsn(){
	return ansn;
  }
  
  /**
   * Returns the time
   * @return the time
   */
  public long getTime(){
	  return time;
  }
  
  /**
   * Sets the time
   * @param time the time
   */
  public void setTime(long time){
	  this.time = time;
  }
  
  /**
   * Method providing a nicely formatted String of the topology tuple
   * @return the nicely formatted topology tuple
   */
  public String toString(){
	  return(destination + " <- " + lastHop);
  }
  

}

