package net.jolsrv2.utils;
/**
 * The class includes helper functions for the encoding/decoding of time in TLVs as described in RFC3626.
 * 
 * @author Andreas Schjnhaug
 *
 */
public class Mantissa {
	
	private static final double C = 0.0625; //Constant C = 1/16
	
	
	/**
	 * The method decodes a byte encoded represented in mantissa/exponent format.
	 * 
	 * time = C*(1+a/16)*2^b (in seconds)
	 * 
	 * where a is the integer represented by the four highest bits of
	 * the byte and b the integer represented by the four lowest
	 * bits of the byte.
	 * 
	 * @param x the encoded byte
	 * @return the time in long
	 */
	public static final long decode(byte x){

		return (long)((C * (1.0 + (double)(x >> 4 & 0xF)/16.0) * Math.pow(2,(x & 0xF)))*(long)1e9);
		
	}
	
	
	/**
	 * The method encodes a double (in seconds) to mantissa/exponent format.
	 * 
	 * @param t the time to encode
	 * @return a byte in mantissa/exponent representation
	 */
	public static final byte encode(double t){
		int b = 0;
		
		while(t/C >= Math.pow(2,b)){
			b++;			
		}
		b--;

		int a = (int)Math.ceil(16*(t/(C*Math.pow(2,b))-1));
		if(a == 16){
			a = 0;
			b++;
		}
		return (byte)(a << 4 | b);
	}
}
